"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ServiceProcess = void 0;
var _events = require("events");
var _stream = require("stream");
var _mergeStream = _interopRequireDefault(require("merge-stream"));
var _split = _interopRequireDefault(require("split2"));
var _spawnProcess = require("./spawnProcess");
var _stream2 = require("./util/stream");
var _processSpawned = require("./util/processSpawned");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
class ServiceProcess {
  logTail = [];
  didError = false;
  didEnd = false;
  wasEndCalled = false;
  constructor(serviceId, serviceConfig, logger, onCrash) {
    this.serviceId = serviceId;
    this.serviceConfig = serviceConfig;
    this.logger = logger;
    this.process = (0, _spawnProcess.spawnProcess)(this.serviceConfig);
    this.started = (0, _processSpawned.processSpawned)(this.process).catch(error => {
      this.didError = true;
      throw error;
    });
    this.output = getProcessOutput(this.process);
    if (this.serviceConfig.logTailLength > 0) {
      this.output = this.output.pipe((0, _stream2.tapStreamLines)(line => {
        this.logTail.push(line);
        if (this.logTail.length > this.serviceConfig.logTailLength) {
          this.logTail.shift();
        }
      }));
    }
    this.ended = (0, _events.once)(this.output, "end").then(() => {
      this.didEnd = true;
    });
    Promise.all([this.started.catch(() => {}), this.ended]).then(() => {
      if (!this.didError && !this.wasEndCalled) {
        onCrash();
      }
    });
  }
  isRunning() {
    return !this.didError && !this.didEnd;
  }
  end(windowsCtrlCShutdown) {
    if (!this.wasEndCalled) {
      this.wasEndCalled = true;
      if (this.isRunning()) {
        if (windowsCtrlCShutdown) {
          // ctrl+c signal was already sent to all service processes
          this.forceKillAfterTimeout();
        } else if (process.platform === "win32") {
          this.process.kill();
        } else {
          this.process.kill("SIGINT");
          this.forceKillAfterTimeout();
        }
      }
    }
    return this.ended;
  }
  forceKillAfterTimeout() {
    if (this.serviceConfig.forceKillTimeout === Infinity) {
      return;
    }
    setTimeout(() => {
      if (this.isRunning()) {
        this.logger.log("info", `Force killing service '${this.serviceId}'`);
        this.process.kill("SIGKILL");
      }
    }, this.serviceConfig.forceKillTimeout);
  }
}
exports.ServiceProcess = ServiceProcess;
function getProcessOutput(proc) {
  return (0, _mergeStream.default)([proc.stdout, proc.stderr].map(stream => stream.setEncoding("utf8")).map(stream => (0, _stream.pipeline)(stream, (0, _split.default)(), (0, _stream2.filterBlankLastLine)(""), () => {})));
}
//# sourceMappingURL=ServiceProcess.js.map