"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.spawnProcess = spawnProcess;
var _child_process = require("child_process");
var _path = require("path");
var _npmRunPath = _interopRequireDefault(require("npm-run-path"));
var _which = _interopRequireDefault(require("which"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Match the isWindows definition from `node-which`
// https://github.com/npm/node-which/blob/6a822d836de79f92fb3170f685a6e283fbfeff87/which.js#L1-L3
const isWindows = process.platform === "win32" || process.env.OSTYPE === "cygwin" || process.env.OSTYPE === "msys";
function spawnProcess(config) {
  const cwd = (0, _path.resolve)(config.cwd);
  let [binary, ...args] = config.command;
  let env = {
    ...config.env
  };
  let path = readEnvCaseInsensitive(env, "PATH") || "";
  path = filterBlankParts((0, _npmRunPath.default)({
    cwd,
    path
  }));
  env = writeEnvCaseNormalized(env, "PATH", path);
  if (isWindows) {
    const pathExt = readEnvCaseInsensitive(env, "PATHEXT") || ".COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH";
    env = writeEnvCaseNormalized(env, "PATHEXT", pathExt);
    /*
      Work around issue (same issue):
        - https://github.com/nodejs/node-v0.x-archive/issues/2318
        - https://github.com/nodejs/node/issues/6671
       Without resorting to wrapping command in shell (like npm package `cross-spawn` does)
      because [spawning a shell is expensive](https://github.com/nodejs/node/issues/6671#issuecomment-219210529)
      and more importantly, that approach introduces *more* window-linux disparities.
       Instead just replace `binary` with a fully-qualified version.
     */
    binary = (0, _path.normalize)(myWhich(cwd, binary, path, pathExt) || binary);
  }
  return (0, _child_process.spawn)(binary, args, {
    cwd,
    env
  });
}
function readEnvCaseInsensitive(env, key) {
  const upperCaseKey = key.toUpperCase();
  const caseInsensitiveKey = Object.keys(env).reverse().find(key => key.toUpperCase() === upperCaseKey);
  return caseInsensitiveKey === undefined ? undefined : env[caseInsensitiveKey];
}
function writeEnvCaseNormalized(env, key, value) {
  const upperCaseKey = key.toUpperCase();
  return {
    ...Object.fromEntries(Object.entries(env).filter(([key]) => key.toUpperCase() !== upperCaseKey)),
    [upperCaseKey]: value
  };
}
function filterBlankParts(string) {
  const colon = isWindows ? ";" : ":";
  return string.split(colon).filter(Boolean).join(colon);
}

// Version of `which.sync` that adds a `cwd` parameter & doesn't throw
function myWhich(cwd, binary, path, pathExt) {
  const originalCwd = process.cwd();
  if (cwd !== originalCwd) {
    process.chdir(cwd);
  }
  const result = _which.default.sync(binary, {
    nothrow: true,
    path,
    pathExt
  });
  if (cwd !== originalCwd) {
    process.chdir(originalCwd);
  }
  return result;
}
//# sourceMappingURL=spawnProcess.js.map